@@ -1,4 +1,4 @@
-use crate::module_loader::import_map::{url_for_path, vega_url, VlVersion};
+use crate::module_loader::import_map::{url_for_path, vega_themes_url, vega_url, VlVersion};
 use crate::module_loader::VlConvertModuleLoader;
 use std::collections::{HashMap, HashSet};
 use std::path::Path;
@@ -16,7 +16,6 @@ use deno_runtime::deno_web::BlobStore;
 use deno_runtime::permissions::Permissions;
 use deno_runtime::worker::MainWorker;
 use deno_runtime::worker::WorkerOptions;
-use deno_runtime::BootstrapOptions;
 
 use std::thread;
 use std::thread::JoinHandle;
@@ -37,6 +36,13 @@ lazy_static! {
     static ref NEXT_ARG_ID: Arc<Mutex<i32>> = Arc::new(Mutex::new(0));
 }
 
+#[derive(Debug, Clone, Default)]
+pub struct VlOpts {
+    pub config: Option<serde_json::Value>,
+    pub theme: Option<String>,
+    pub vl_version: VlVersion,
+}
+
 fn set_json_arg(arg: serde_json::Value) -> Result<i32, AnyError> {
     // Increment arg id
     let id = match NEXT_ARG_ID.lock() {
@@ -99,8 +105,15 @@ impl InnerVlConverter {
 var vega;
 import('{vega_url}').then((imported) => {{
     vega = imported;
-}})"#,
-                vega_url = vega_url()
+}})
+
+var vegaThemes;
+import('{vega_themes_url}').then((imported) => {{
+    vegaThemes = imported;
+}})
+"#,
+                vega_url = vega_url(),
+                vega_themes_url = vega_themes_url(),
             );
 
             self.worker.execute_script("<anon>", &import_str)?;
@@ -174,13 +187,33 @@ import('{vl_url}').then((imported) => {{
             // Create and initialize function string
             let function_str = format!(
                 r#"
-function compileVegaLite_{ver_name}(vlSpec) {{
+function compileVegaLite_{ver_name}(vlSpec, config, theme) {{
     let options = {{}};
+
+    // Handle config and theme
+    let usermetaTheme = ((vlSpec.usermeta ?? {{}}).embedOptions ?? {{}}).theme;
+    let namedTheme = theme ?? usermetaTheme;
+    if (namedTheme != null) {{
+        options["config"] = vega.mergeConfig(vegaThemes[namedTheme], config ?? {{}});
+    }} else if (config != null) {{
+        options["config"] = config;
+    }}
+
     return {ver_name}.compile(vlSpec, options).spec
 }}
 
-function vegaLiteToSvg_{ver_name}(vlSpec) {{
+function vegaLiteToSvg_{ver_name}(vlSpec, config, theme) {{
     let options = {{}};
+
+    // Handle config and theme
+    let usermetaTheme = ((vlSpec.usermeta ?? {{}}).embedOptions ?? {{}}).theme;
+    let namedTheme = theme ?? usermetaTheme;
+    if (namedTheme != null) {{
+        options["config"] = vega.mergeConfig(vegaThemes[namedTheme], config ?? {{}});
+    }} else if (config != null) {{
+        options["config"] = config;
+    }}
+
     let vgSpec = {ver_name}.compile(vlSpec, options).spec;
     return vegaToSvg(vgSpec)
 }}
@@ -217,21 +250,9 @@ function vegaLiteToSvg_{ver_name}(vlSpec) {{
         });
 
         let options = WorkerOptions {
-            bootstrap: BootstrapOptions {
-                args: vec![],
-                cpu_count: 1,
-                debug_flag: false,
-                enable_testing_features: false,
-                location: None,
-                no_color: false,
-                is_tty: false,
-                runtime_version: "x".to_string(),
-                ts_version: "x".to_string(),
-                unstable: false,
-                user_agent: "hello_runtime".to_string(),
-                inspect: false,
-            },
+            bootstrap: Default::default(),
             extensions: vec![ext],
+            startup_snapshot: None,
             unsafely_ignore_certificate_errors: None,
             root_cert_store: None,
             seed: None,
@@ -252,6 +273,7 @@ function vegaLiteToSvg_{ver_name}(vlSpec) {{
             shared_array_buffer_store: None,
             compiled_wasm_module_store: None,
             stdio: Default::default(),
+            should_wait_for_inspector_session: false,
         };
 
         let js_path = Path::new(env!("CARGO_MANIFEST_DIR")).join("vl-convert-rs.js");
@@ -321,19 +343,32 @@ function vegaLiteToSvg_{ver_name}(vlSpec) {{
     pub async fn vegalite_to_vega(
         &mut self,
         vl_spec: &serde_json::Value,
-        vl_version: VlVersion,
+        vl_opts: VlOpts,
     ) -> Result<serde_json::Value, AnyError> {
-        self.init_vl_version(&vl_version).await?;
+        self.init_vega().await?;
+        self.init_vl_version(&vl_opts.vl_version).await?;
+        let config = vl_opts.config.clone().unwrap_or(serde_json::Value::Null);
+
+        let spec_arg_id = set_json_arg(vl_spec.clone())?;
+        let config_arg_id = set_json_arg(config)?;
+
+        let theme_arg = match &vl_opts.theme {
+            None => "null".to_string(),
+            Some(s) => format!("'{}'", s),
+        };
 
-        let arg_id = set_json_arg(vl_spec.clone())?;
         let code = format!(
             r#"
 compileVegaLite_{ver_name:?}(
-    JSON.parse(Deno.core.ops.op_get_json_arg({arg_id}))
+    JSON.parse(Deno.core.ops.op_get_json_arg({spec_arg_id})),
+    JSON.parse(Deno.core.ops.op_get_json_arg({config_arg_id})),
+    {theme_arg}
 )
 "#,
-            ver_name = vl_version,
-            arg_id = arg_id
+            ver_name = vl_opts.vl_version,
+            spec_arg_id = spec_arg_id,
+            config_arg_id = config_arg_id,
+            theme_arg = theme_arg,
         );
 
         let value = self.execute_script_to_json(&code).await?;
@@ -343,23 +378,34 @@ compileVegaLite_{ver_name:?}(
     pub async fn vegalite_to_svg(
         &mut self,
         vl_spec: &serde_json::Value,
-        vl_version: VlVersion,
+        vl_opts: VlOpts,
     ) -> Result<String, AnyError> {
         self.init_vega().await?;
-        self.init_vl_version(&vl_version).await?;
+        self.init_vl_version(&vl_opts.vl_version).await?;
+
+        let config = vl_opts.config.clone().unwrap_or(serde_json::Value::Null);
+        let spec_arg_id = set_json_arg(vl_spec.clone())?;
+        let config_arg_id = set_json_arg(config)?;
+        let theme_arg = match &vl_opts.theme {
+            None => "null".to_string(),
+            Some(s) => format!("'{}'", s),
+        };
 
-        let arg_id = set_json_arg(vl_spec.clone())?;
         let code = format!(
             r#"
 var svg;
 vegaLiteToSvg_{ver_name:?}(
-    JSON.parse(Deno.core.ops.op_get_json_arg({arg_id}))
+    JSON.parse(Deno.core.ops.op_get_json_arg({spec_arg_id})),
+    JSON.parse(Deno.core.ops.op_get_json_arg({config_arg_id})),
+    {theme_arg}
 ).then((result) => {{
     svg = result;
 }});
 "#,
-            ver_name = vl_version,
-            arg_id = arg_id,
+            ver_name = vl_opts.vl_version,
+            spec_arg_id = spec_arg_id,
+            config_arg_id = config_arg_id,
+            theme_arg = theme_arg,
         );
         self.worker.execute_script("<anon>", &code)?;
         self.worker.run_event_loop(false).await?;
@@ -390,12 +436,31 @@ vegaToSvg(
         Ok(value)
     }
 
-    pub async fn get_local_tz(&mut self) -> Result<String, AnyError> {
-        let code = "var localTz = Intl.DateTimeFormat().resolvedOptions().timeZone;";
+    pub async fn get_local_tz(&mut self) -> Result<Option<String>, AnyError> {
+        let code = "var localTz = Intl.DateTimeFormat().resolvedOptions().timeZone ?? 'undefined';";
         self.worker.execute_script("<anon>", code)?;
         self.worker.run_event_loop(false).await?;
 
         let value = self.execute_script_to_string("localTz").await?;
+        if value == "undefined" {
+            Ok(None)
+        } else {
+            Ok(Some(value))
+        }
+    }
+
+    pub async fn get_themes(&mut self) -> Result<serde_json::Value, AnyError> {
+        self.init_vega().await?;
+
+        let code = r#"
+var themes = Object.assign({}, vegaThemes);
+delete themes.version
+delete themes.default
+"#;
+        self.worker.execute_script("<anon>", code)?;
+        self.worker.run_event_loop(false).await?;
+
+        let value = self.execute_script_to_json("themes").await?;
         Ok(value)
     }
 }
@@ -403,7 +468,7 @@ vegaToSvg(
 pub enum VlConvertCommand {
     VlToVg {
         vl_spec: serde_json::Value,
-        vl_version: VlVersion,
+        vl_opts: VlOpts,
         responder: oneshot::Sender<Result<serde_json::Value, AnyError>>,
     },
     VgToSvg {
@@ -412,11 +477,14 @@ pub enum VlConvertCommand {
     },
     VlToSvg {
         vl_spec: serde_json::Value,
-        vl_version: VlVersion,
+        vl_opts: VlOpts,
         responder: oneshot::Sender<Result<String, AnyError>>,
     },
     GetLocalTz {
-        responder: oneshot::Sender<Result<String, AnyError>>,
+        responder: oneshot::Sender<Result<Option<String>, AnyError>>,
+    },
+    GetThemes {
+        responder: oneshot::Sender<Result<serde_json::Value, AnyError>>,
     },
 }
 
@@ -447,7 +515,7 @@ pub enum VlConvertCommand {
 /// }   "#).unwrap();
 ///
 ///     let vega_spec = futures::executor::block_on(
-///         converter.vegalite_to_vega(vl_spec, VlVersion::v5_5)
+///         converter.vegalite_to_vega(vl_spec, Default::default())
 ///     ).expect(
 ///         "Failed to perform Vega-Lite to Vega conversion"
 ///     );
@@ -471,11 +539,11 @@ impl VlConverter {
                 match cmd {
                     VlConvertCommand::VlToVg {
                         vl_spec,
-                        vl_version,
+                        vl_opts,
                         responder,
                     } => {
                         let vega_spec =
-                            TOKIO_RUNTIME.block_on(inner.vegalite_to_vega(&vl_spec, vl_version));
+                            TOKIO_RUNTIME.block_on(inner.vegalite_to_vega(&vl_spec, vl_opts));
                         responder.send(vega_spec).ok();
                     }
                     VlConvertCommand::VgToSvg { vg_spec, responder } => {
@@ -484,17 +552,21 @@ impl VlConverter {
                     }
                     VlConvertCommand::VlToSvg {
                         vl_spec,
-                        vl_version,
+                        vl_opts,
                         responder,
                     } => {
                         let svg_result =
-                            TOKIO_RUNTIME.block_on(inner.vegalite_to_svg(&vl_spec, vl_version));
+                            TOKIO_RUNTIME.block_on(inner.vegalite_to_svg(&vl_spec, vl_opts));
                         responder.send(svg_result).ok();
                     }
                     VlConvertCommand::GetLocalTz { responder } => {
                         let local_tz = TOKIO_RUNTIME.block_on(inner.get_local_tz());
                         responder.send(local_tz).ok();
                     }
+                    VlConvertCommand::GetThemes { responder } => {
+                        let themes = TOKIO_RUNTIME.block_on(inner.get_themes());
+                        responder.send(themes).ok();
+                    }
                 }
             }
             Ok(())
@@ -509,12 +581,12 @@ impl VlConverter {
     pub async fn vegalite_to_vega(
         &mut self,
         vl_spec: serde_json::Value,
-        vl_version: VlVersion,
+        vl_opts: VlOpts,
     ) -> Result<serde_json::Value, AnyError> {
         let (resp_tx, resp_rx) = oneshot::channel::<Result<serde_json::Value, AnyError>>();
         let cmd = VlConvertCommand::VlToVg {
             vl_spec,
-            vl_version,
+            vl_opts,
             responder: resp_tx,
         };
 
@@ -562,12 +634,12 @@ impl VlConverter {
     pub async fn vegalite_to_svg(
         &mut self,
         vl_spec: serde_json::Value,
-        vl_version: VlVersion,
+        vl_opts: VlOpts,
     ) -> Result<String, AnyError> {
         let (resp_tx, resp_rx) = oneshot::channel::<Result<String, AnyError>>();
         let cmd = VlConvertCommand::VlToSvg {
             vl_spec,
-            vl_version,
+            vl_opts,
             responder: resp_tx,
         };
 
@@ -601,11 +673,11 @@ impl VlConverter {
     pub async fn vegalite_to_png(
         &mut self,
         vl_spec: serde_json::Value,
-        vl_version: VlVersion,
+        vl_opts: VlOpts,
         scale: Option<f32>,
     ) -> Result<Vec<u8>, AnyError> {
         let scale = scale.unwrap_or(1.0);
-        let svg = self.vegalite_to_svg(vl_spec, vl_version).await?;
+        let svg = self.vegalite_to_svg(vl_spec, vl_opts).await?;
         Self::svg_to_png(&svg, scale)
     }
 
@@ -620,7 +692,7 @@ impl VlConverter {
             }
         };
 
-        let pixmap_size = rtree.svg_node().size.to_screen_size();
+        let pixmap_size = rtree.size.to_screen_size();
         let mut pixmap = tiny_skia::Pixmap::new(
             (pixmap_size.width() as f32 * scale) as u32,
             (pixmap_size.height() as f32 * scale) as u32,
@@ -642,8 +714,8 @@ impl VlConverter {
         }
     }
 
-    pub async fn get_local_tz(&mut self) -> Result<String, AnyError> {
-        let (resp_tx, resp_rx) = oneshot::channel::<Result<String, AnyError>>();
+    pub async fn get_local_tz(&mut self) -> Result<Option<String>, AnyError> {
+        let (resp_tx, resp_rx) = oneshot::channel::<Result<Option<String>, AnyError>>();
         let cmd = VlConvertCommand::GetLocalTz { responder: resp_tx };
 
         // Send request
@@ -665,6 +737,27 @@ impl VlConverter {
             ),
         }
     }
+
+    pub async fn get_themes(&mut self) -> Result<serde_json::Value, AnyError> {
+        let (resp_tx, resp_rx) = oneshot::channel::<Result<serde_json::Value, AnyError>>();
+        let cmd = VlConvertCommand::GetThemes { responder: resp_tx };
+
+        // Send request
+        match self.sender.send(cmd).await {
+            Ok(_) => {
+                // All good
+            }
+            Err(err) => {
+                bail!("Failed to send get_themes request: {}", err.to_string())
+            }
+        }
+
+        // Wait for result
+        match resp_rx.await {
+            Ok(themes_result) => themes_result,
+            Err(err) => bail!("Failed to retrieve get_themes result: {}", err.to_string()),
+        }
+    }
 }
 
 impl Default for VlConverter {
@@ -692,7 +785,13 @@ mod tests {
         "##).unwrap();
 
         let vg_spec = ctx
-            .vegalite_to_vega(vl_spec, VlVersion::v4_17)
+            .vegalite_to_vega(
+                vl_spec,
+                VlOpts {
+                    vl_version: VlVersion::v4_17,
+                    ..Default::default()
+                },
+            )
             .await
             .unwrap();
         println!("vg_spec: {}", vg_spec)
@@ -713,14 +812,26 @@ mod tests {
 
         let mut ctx1 = VlConverter::new();
         let vg_spec1 = ctx1
-            .vegalite_to_vega(vl_spec.clone(), VlVersion::v4_17)
+            .vegalite_to_vega(
+                vl_spec.clone(),
+                VlOpts {
+                    vl_version: VlVersion::v4_17,
+                    ..Default::default()
+                },
+            )
             .await
             .unwrap();
         println!("vg_spec1: {}", vg_spec1);
 
         let mut ctx1 = VlConverter::new();
         let vg_spec2 = ctx1
-            .vegalite_to_vega(vl_spec, VlVersion::v5_5)
+            .vegalite_to_vega(
+                vl_spec,
+                VlOpts {
+                    vl_version: VlVersion::v5_5,
+                    ..Default::default()
+                },
+            )
             .await
             .unwrap();
         println!("vg_spec2: {}", vg_spec2);
