@@ -4,7 +4,7 @@ use pyo3::types::{PyBytes, PyDict};
 use pythonize::{depythonize, pythonize};
 use std::str::FromStr;
 use std::sync::Mutex;
-use vl_convert_rs::converter::TOKIO_RUNTIME;
+use vl_convert_rs::converter::{VlOpts, TOKIO_RUNTIME};
 use vl_convert_rs::module_loader::import_map::VlVersion;
 use vl_convert_rs::serde_json;
 use vl_convert_rs::text::register_font_directory as register_font_directory_rs;
@@ -22,17 +22,22 @@ lazy_static! {
 ///
 /// Args:
 ///     vl_spec (str | dict): Vega-Lite JSON specification string or dict
-///     vl_version (str): Vega-Lite library version string (e.g. 'v5.5')
+///     vl_version (str | None): Vega-Lite library version string (e.g. 'v5.5')
 ///         (default to latest)
-///     pretty (bool): If True, pretty-print resulting Vega JSON
-///         specification (default False)
-///
+///     config (dict | None): Chart configuration object to apply during conversion
+///     theme (str | None): Named theme (e.g. "dark") to apply during conversion
 /// Returns:
-///     str: Vega JSON specification string
+///     dict: Vega JSON specification dict
 #[pyfunction]
-#[pyo3(text_signature = "(vl_spec, vl_version, pretty)")]
-fn vegalite_to_vega(vl_spec: PyObject, vl_version: Option<&str>) -> PyResult<PyObject> {
+#[pyo3(text_signature = "(vl_spec, vl_version, config, theme)")]
+fn vegalite_to_vega(
+    vl_spec: PyObject,
+    vl_version: Option<&str>,
+    config: Option<PyObject>,
+    theme: Option<String>,
+) -> PyResult<PyObject> {
     let vl_spec = parse_json_spec(vl_spec)?;
+    let config = config.and_then(|c| parse_json_spec(c).ok());
 
     let vl_version = if let Some(vl_version) = vl_version {
         VlVersion::from_str(vl_version)?
@@ -43,7 +48,14 @@ fn vegalite_to_vega(vl_spec: PyObject, vl_version: Option<&str>) -> PyResult<PyO
     let mut converter = VL_CONVERTER
         .lock()
         .expect("Failed to acquire lock on Vega-Lite converter");
-    let vega_spec = match TOKIO_RUNTIME.block_on(converter.vegalite_to_vega(vl_spec, vl_version)) {
+    let vega_spec = match TOKIO_RUNTIME.block_on(converter.vegalite_to_vega(
+        vl_spec,
+        VlOpts {
+            vl_version,
+            config,
+            theme,
+        },
+    )) {
         Ok(vega_spec) => vega_spec,
         Err(err) => {
             return Err(PyValueError::new_err(format!(
@@ -90,15 +102,22 @@ fn vega_to_svg(vg_spec: PyObject) -> PyResult<String> {
 ///
 /// Args:
 ///     vl_spec (str | dict): Vega-Lite JSON specification string or dict
-///     vl_version (str): Vega-Lite library version string (e.g. 'v5.5')
+///     vl_version (str | None): Vega-Lite library version string (e.g. 'v5.5')
 ///         (default to latest)
-///
+///     config (dict | None): Chart configuration object to apply during conversion
+///     theme (str | None): Named theme (e.g. "dark") to apply during conversion
 /// Returns:
 ///     str: SVG image string
 #[pyfunction]
-#[pyo3(text_signature = "(vl_spec, vl_version)")]
-fn vegalite_to_svg(vl_spec: PyObject, vl_version: Option<&str>) -> PyResult<String> {
+#[pyo3(text_signature = "(vl_spec, vl_version, config, theme)")]
+fn vegalite_to_svg(
+    vl_spec: PyObject,
+    vl_version: Option<&str>,
+    config: Option<PyObject>,
+    theme: Option<String>,
+) -> PyResult<String> {
     let vl_spec = parse_json_spec(vl_spec)?;
+    let config = config.and_then(|c| parse_json_spec(c).ok());
 
     let vl_version = if let Some(vl_version) = vl_version {
         VlVersion::from_str(vl_version)?
@@ -110,7 +129,14 @@ fn vegalite_to_svg(vl_spec: PyObject, vl_version: Option<&str>) -> PyResult<Stri
         .lock()
         .expect("Failed to acquire lock on Vega-Lite converter");
 
-    let svg = match TOKIO_RUNTIME.block_on(converter.vegalite_to_svg(vl_spec, vl_version)) {
+    let svg = match TOKIO_RUNTIME.block_on(converter.vegalite_to_svg(
+        vl_spec,
+        VlOpts {
+            vl_version,
+            config,
+            theme,
+        },
+    )) {
         Ok(vega_spec) => vega_spec,
         Err(err) => {
             return Err(PyValueError::new_err(format!(
@@ -162,15 +188,19 @@ fn vega_to_png(vg_spec: PyObject, scale: Option<f32>) -> PyResult<PyObject> {
 ///     vl_version (str): Vega-Lite library version string (e.g. 'v5.5')
 ///         (default to latest)
 ///     scale (float): Image scale factor (default 1.0)
+///     config (dict | None): Chart configuration object to apply during conversion
+///     theme (str | None): Named theme (e.g. "dark") to apply during conversion
 ///
 /// Returns:
 ///     bytes: PNG image data
 #[pyfunction]
-#[pyo3(text_signature = "(vl_spec, vl_version, scale)")]
+#[pyo3(text_signature = "(vl_spec, vl_version, scale, config, theme)")]
 fn vegalite_to_png(
     vl_spec: PyObject,
     vl_version: Option<&str>,
     scale: Option<f32>,
+    config: Option<PyObject>,
+    theme: Option<String>,
 ) -> PyResult<PyObject> {
     let vl_version = if let Some(vl_version) = vl_version {
         VlVersion::from_str(vl_version)?
@@ -178,21 +208,29 @@ fn vegalite_to_png(
         Default::default()
     };
     let vl_spec = parse_json_spec(vl_spec)?;
+    let config = config.and_then(|c| parse_json_spec(c).ok());
 
     let mut converter = VL_CONVERTER
         .lock()
         .expect("Failed to acquire lock on Vega-Lite converter");
 
-    let png_data =
-        match TOKIO_RUNTIME.block_on(converter.vegalite_to_png(vl_spec, vl_version, scale)) {
-            Ok(vega_spec) => vega_spec,
-            Err(err) => {
-                return Err(PyValueError::new_err(format!(
-                    "Vega to SVG conversion failed:\n{}",
-                    err
-                )))
-            }
-        };
+    let png_data = match TOKIO_RUNTIME.block_on(converter.vegalite_to_png(
+        vl_spec,
+        VlOpts {
+            vl_version,
+            config,
+            theme,
+        },
+        scale,
+    )) {
+        Ok(vega_spec) => vega_spec,
+        Err(err) => {
+            return Err(PyValueError::new_err(format!(
+                "Vega to SVG conversion failed:\n{}",
+                err
+            )))
+        }
+    };
 
     Ok(Python::with_gil(|py| -> PyObject {
         PyObject::from(PyBytes::new(py, png_data.as_slice()))
@@ -243,10 +281,11 @@ fn register_font_directory(font_dir: &str) -> PyResult<()> {
 /// Get the named local timezone that Vega uses to perform timezone calculations
 ///
 /// Returns:
-///     str: Named local timezone (e.g. "America/New_York")
+///     str: Named local timezone (e.g. "America/New_York"),
+///          or None if the local timezone cannot be determined
 #[pyfunction]
 #[pyo3(text_signature = "()")]
-fn get_local_tz() -> PyResult<String> {
+fn get_local_tz() -> PyResult<Option<String>> {
     let mut converter = VL_CONVERTER
         .lock()
         .expect("Failed to acquire lock on Vega-Lite converter");
@@ -262,6 +301,30 @@ fn get_local_tz() -> PyResult<String> {
     Ok(local_tz)
 }
 
+/// Get the config dict for each built-in theme
+///
+/// Returns:
+///     dict: dict from theme name to config object
+#[pyfunction]
+#[pyo3(text_signature = "()")]
+fn get_themes() -> PyResult<PyObject> {
+    let mut converter = VL_CONVERTER
+        .lock()
+        .expect("Failed to acquire lock on Vega-Lite converter");
+    let themes = match TOKIO_RUNTIME.block_on(converter.get_themes()) {
+        Ok(themes) => themes,
+        Err(err) => {
+            return Err(PyValueError::new_err(format!(
+                "get_themes request failed:\n{}",
+                err
+            )))
+        }
+    };
+    Python::with_gil(|py| -> PyResult<PyObject> {
+        pythonize(py, &themes).map_err(|err| PyValueError::new_err(err.to_string()))
+    })
+}
+
 /// Convert Vega-Lite specifications to other formats
 #[pymodule]
 fn vl_convert(_py: Python, m: &PyModule) -> PyResult<()> {
@@ -272,6 +335,7 @@ fn vl_convert(_py: Python, m: &PyModule) -> PyResult<()> {
     m.add_function(wrap_pyfunction!(vega_to_png, m)?)?;
     m.add_function(wrap_pyfunction!(register_font_directory, m)?)?;
     m.add_function(wrap_pyfunction!(get_local_tz, m)?)?;
+    m.add_function(wrap_pyfunction!(get_themes, m)?)?;
     m.add("__version__", env!("CARGO_PKG_VERSION"))?;
     Ok(())
 }
